/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2001
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <cups/cups.h>
#include <cups/raster.h>

#include "Device.hpp"
#include "StringResource.hpp"

#define VERSION      "0.0.6.2"

void
outputPostscriptName (DeviceForm *pDF)
{
   HardCopyCap *pHCC = pDF->getHardCopyCap ();
   int          iCx  = (int)(72.0 * (float)pHCC->getCx () / 2540.0);;
   int          iCy  = (int)(72.0 * (float)pHCC->getCy () / 2540.0);;

   switch (pDF->getID ())
   {
   case DeviceForm::FORM_LETTER:           cout << "Letter"; break;
   case DeviceForm::FORM_LEGAL:            cout << "Legal"; break;
   case DeviceForm::FORM_EXECUTIVE:        cout << "Executive"; break;
   case DeviceForm::FORM_TABLOID:          cout << "Tabloid"; break;
   case DeviceForm::FORM_A0:               cout << "A0"; break;
   case DeviceForm::FORM_A1:               cout << "A1"; break;
   case DeviceForm::FORM_A2:               cout << "A2"; break;
   case DeviceForm::FORM_A3:               cout << "A3"; break;
   case DeviceForm::FORM_A4:               cout << "A4"; break;
   case DeviceForm::FORM_A5:               cout << "A5"; break;
   case DeviceForm::FORM_A6:               cout << "A6"; break;
   case DeviceForm::FORM_B0:               cout << "B0"; break;
   case DeviceForm::FORM_B1:               cout << "B1"; break;
   case DeviceForm::FORM_B2:               cout << "B2"; break;
   case DeviceForm::FORM_B3:               cout << "B3"; break;
   case DeviceForm::FORM_B4:               cout << "B4"; break;
   case DeviceForm::FORM_B5:               cout << "B5"; break;
   case DeviceForm::FORM_C5_ENVELOPE:      cout << "EnvC5"; break;
   case DeviceForm::FORM_DL_ENVELOPE:      cout << "EnvDL"; break;
   case DeviceForm::FORM_MONARCH_ENVELOPE: cout << "EnvMonarch"; break;
   default:
   {
      cout << "w"
           << iCx
           << "h"
           << iCy;
      break;
   }
   }
}

int
main (int argc, char *argv[])
{
   char          achLibDeviceName[512];
   char         *pszDeviceName;
   void         *vhDevice              = 0;
   Device       *pDevice               = 0;
   PFNNEWDEVICE  pfnNewDevice          = 0;

   if (2 != argc)
   {
      cerr << "Usage: " << argv[0] << " printer_library_name" << endl;
      return 1;
   }

   if (0 != strncasecmp (argv[1], "lib", 3))
   {
      sprintf (achLibDeviceName, "lib%s.so", argv[1]);
   }
   else
   {
      strcpy (achLibDeviceName, argv[1]);
   }

   cerr << "Trying to load " << achLibDeviceName << endl;

   vhDevice = dlopen (achLibDeviceName, RTLD_NOW | RTLD_GLOBAL);

   cerr << "vhDevice = " << hex << vhDevice << endl;

   if (!vhDevice)
   {
      cerr << "dlerror returns " << dlerror () << endl;
      return 2;
   }

   pfnNewDevice = (PFNNEWDEVICE)dlsym (vhDevice, "newDevice__Fb");

   cerr << hex << "pfnNewDevice = 0x" << (int)pfnNewDevice << endl;

   if (!pfnNewDevice)
   {
      cerr << "Error: " << achLibDeviceName << " is not a valid library" << endl;

      dlclose (vhDevice);

      return 3;
   }

   pszDeviceName = achLibDeviceName + 3; // Skip the lib part

   char *pszDot = strchr (pszDeviceName, '.');
   pszDot[0] = '\0';

   pDevice = pfnNewDevice (true);

   if (!pDevice)
   {
      cerr << "Could not instantiate a new device." << endl;

      dlclose (vhDevice);

      return 4;
   }

   cerr << "pDevice = " << *pDevice << endl;

   cout << "*PPD-Adobe: \"4.3\"" << endl;
   cout << "*%PPD file for CUPS/Omni." << endl;
   cout << "*%This PPD file may be freely used and distributed under the terms of" << endl;
   cout << "*%the GNU LGPL." << endl;
   cout << "*FormatVersion:	\"4.3\"" << endl;
   cout << "*FileVersion:	\"" VERSION "\"" << endl;
   cout << "*LanguageVersion: English" << endl;
   cout << "*LanguageEncoding: ISOLatin1" << endl;
   cout << "*PCFileName:	\"" << pszDeviceName << ".ppd\"" << endl;
   cout << "*Manufacturer:	\"" << pDevice->getDriverName () << "\"" << endl;
   cout << "*Product:	\"(Omni v" << pDevice->getVersion () << ")\"" << endl;
   cout << "*ModelName:	\"" << pDevice->getDeviceName () << "\"" << endl;
// *ShortNickName @TBD
   cout << "*NickName	\"lib" << pszDeviceName << ".so\"" << endl; // @TBD
   cout << "*PSVersion:	\"(3010.000) 550\"" << endl;
   cout << "*LanguageLevel:	\"3\"" << endl;

   if (pDevice->hasCapability (Capability::COLOR))
   {
      cout << "*ColorDevice:	True" << endl;
      cout << "*DefaultColorSpace:	RGB" << endl;
   }
   else
   {
      cout << "*ColorDevice:	False" << endl;
      cout << "*DefaultColorSpace:	Gray" << endl;
   }

   cout << "*FileSystem:	False" << endl;
   cout << "*LandscapeOrientation: Plus90" << endl;
   cout << "*TTRasterizer:	Type42" << endl;

   cout << "*cupsVersion:	1.1" << endl;
// *cupsModelNumber @TBD
   cout << "*cupsManualCopies: True" << endl;
   cout << "*cupsFilter:	\"application/vnd.cups-raster 100 CUPSToOmni\"" << endl;
// EPSON @TBD
// *cupsFilter:	\"application/vnd.cups-command 100 commandtoepson\"\n"); @TBD

   cout << "*OpenUI *PageSize: PickOne" << endl;
   cout << "*OrderDependency: 10 AnySetup *PageSize" << endl;

   DeviceForm  *pDF   = pDevice->getCurrentForm ();
   Enumeration *pEnum = pDF->getEnumeration ();

   cout << "*DefaultPageSize: ";
   outputPostscriptName (pDF);
   cout << endl;

   while (pEnum->hasMoreElements ())
   {
      pDF = (DeviceForm *)pEnum->nextElement ();

      HardCopyCap *pHCC = pDF->getHardCopyCap ();
      int          iCx;
      int          iCy;

      iCx = (int)(72.0 * (float)pHCC->getCx () / 2540.0);
      iCy = (int)(72.0 * (float)pHCC->getCy () / 2540.0);

      cout << "*PageSize ";

      outputPostscriptName (pDF);

      cout << "/"
           << pDF->getName ()
           << ":\t\"<</PageSize["
           << iCx
           << " "
           << iCy
           << "]/ImagingBBox null>>setpagedevice\""
           << endl;

      delete pDF;
   }

   cout << "*CloseUI: *PageSize" << endl;

   delete pEnum;

   cout << "*OpenUI *PageRegion: PickOne" << endl;
   cout << "*OrderDependency: 10 AnySetup *PageSize" << endl;

   pDF   = pDevice->getCurrentForm ();
   pEnum = pDF->getEnumeration ();

   cout << "*DefaultPageRegion: ";
   outputPostscriptName (pDF);
   cout << endl;

   while (pEnum->hasMoreElements ())
   {
      pDF = (DeviceForm *)pEnum->nextElement ();

      HardCopyCap *pHCC = pDF->getHardCopyCap ();
      int          iCx;
      int          iCy;

      iCx = (int)(72.0 * (float)pHCC->getCx () / 2540.0);
      iCy = (int)(72.0 * (float)pHCC->getCy () / 2540.0);

      cout << "*PageRegion ";

      outputPostscriptName (pDF);

      cout << "/"
           << pDF->getName ()
           << ":\t\"<</PageRegion["
           << iCx
           << " "
           << iCy
           << "]/ImagingBBox null>>setpagedevice\""
           << endl;

      delete pDF;
   }

   cout << "*CloseUI: *PageRegion" << endl;

   delete pEnum;

   pDF   = pDevice->getCurrentForm ();
   pEnum = pDF->getEnumeration ();

   cout << "*DefaultImageableArea: ";
   outputPostscriptName (pDF);
   cout << endl;

   while (pEnum->hasMoreElements ())
   {
      pDF = (DeviceForm *)pEnum->nextElement ();

      HardCopyCap *pHCC    = pDF->getHardCopyCap ();
      int          iCx     = (int)(72.0 * (float)pHCC->getCx         () / 2540.0);
      int          iCy     = (int)(72.0 * (float)pHCC->getCy         () / 2540.0);
      int          iLeft   = (int)(72.0 * (float)pHCC->getLeftClip   () / 2540.0);
      int          iRight  = (int)(72.0 * (float)pHCC->getRightClip  () / 2540.0);
      int          iTop    = (int)(72.0 * (float)pHCC->getTopClip    () / 2540.0);
      int          iBottom = (int)(72.0 * (float)pHCC->getBottomClip () / 2540.0);

      cout << "*ImageableArea ";

      outputPostscriptName (pDF);

      cout << "/"
           << pDF->getName ()
           << ":\t\""
           << iLeft
           << " "
           << iBottom
           << " "
           << iCx - iRight
           << " "
           << iCy - iTop
           << "\""
           << endl;

      delete pDF;
   }

   delete pEnum;

   pDF   = pDevice->getCurrentForm ();
   pEnum = pDF->getEnumeration ();

   cout << "*DefaultPaperDimension: ";
   outputPostscriptName (pDF);
   cout << endl;

   while (pEnum->hasMoreElements ())
   {
      pDF = (DeviceForm *)pEnum->nextElement ();

      HardCopyCap *pHCC    = pDF->getHardCopyCap ();
      int          iCx     = (int)(72.0 * (float)pHCC->getCx () / 2540.0);
      int          iCy     = (int)(72.0 * (float)pHCC->getCy () / 2540.0);

      cout << "*PaperDimension ";

      outputPostscriptName (pDF);

      cout << "/"
           << pDF->getName ()
           << ":\t\""
           << iCx
           << " "
           << iCy
           << "\""
           << endl;

      delete pDF;
   }

   delete pEnum;

   cout << "*OpenUI *ColorModel: PickOne" << endl;
   cout << "*OrderDependency: 10 AnySetup *ColorModel" << endl;

   if (pDevice->hasCapability (Capability::COLOR))
   {
      cout << "*DefaultColorModel: RGB" << endl;
      cout << "*ColorModel Gray/Grayscale:\t\"<</cupsColorSpace "
           << (unsigned long int)CUPS_CSPACE_W
           << "/cupsColorOrder "
           << (unsigned long int)CUPS_ORDER_CHUNKED
           << "/cupsBitsPerColor 8>>setpagedevice\""
           << endl;
      cout << "*ColorModel RGB/Color:\t\"<</cupsColorSpace "
           << (unsigned long int)CUPS_CSPACE_RGB
           << "/cupsColorOrder "
           << (unsigned long int)CUPS_ORDER_CHUNKED
           << "/cupsBitsPerColor 8>>setpagedevice\"" // @TBD - shouldn't this be 24 bits per pel?
           << endl;
   }
   else
   {
      cout << "*DefaultColorModel: Gray" << endl;
      cout << "*ColorModel Gray/Grayscale:\t\"<</cupsColorSpace "
           << (unsigned long int)CUPS_CSPACE_W
           << "/cupsColorOrder "
           << (unsigned long int)CUPS_ORDER_CHUNKED
           << "/cupsBitsPerColor 8>>setpagedevice\"" // @TBD - shouldn't this be 1 bit per pel?
           << endl;
   }

   cout << "*CloseUI: *ColorModel" << endl;

   cout << "*OpenUI *MediaType: PickOne" << endl;
   cout << "*OrderDependency: 10 AnySetup *MediaType" << endl;

   DeviceMedia *pDM = pDevice->getCurrentMedia ();

   cout << "*DefaultMediaType: " << DeviceMedia::IDToName (pDM->getID ()) << endl;

   pEnum = pDM->getEnumeration ();

   while (pEnum->hasMoreElements ())
   {
      pDM = (DeviceMedia *)pEnum->nextElement ();

      cout << "*MediaType "
           << DeviceMedia::IDToName (pDM->getID ())
           << "/"
           << pDM->getName ()
           << ":\t\"<</MediaType("
           << pDM->getName ()        // @TBD - should this be our settable name?
           << ")>>setpagedevice\""
           << endl;

      delete pDM;
   }

   cout << "*CloseUI: *MediaType" << endl;

   delete pEnum;

   cout << "*OpenUI *InputSlot: PickOne" << endl;
   cout << "*OrderDependency: 10 AnySetup *InputSlot" << endl;

   DeviceTray *pDT = pDevice->getCurrentTray ();

   cout << "*DefaultInputSlot: " << DeviceTray::IDToName (pDT->getID ()) << endl;

   pEnum = pDT->getEnumeration ();

   while (pEnum->hasMoreElements ())
   {
      pDT = (DeviceTray *)pEnum->nextElement ();

      cout << "*InputSlot "
           << DeviceTray::IDToName (pDT->getID ())
           << "/"
           << pDT->getName ()
           << ":\t\"<</MediaClass("
           << pDT->getName ()        // @TBD - should this be our settable name?
           << ")>>setpagedevice\""
           << endl;

      delete pDT;
   }

   cout << "*CloseUI: *InputSlot" << endl;

   delete pEnum;

   cout << "*OpenUI *Dither: PickOne" << endl;
   cout << "*OrderDependency: 10 AnySetup *Dither" << endl;

   DeviceDither *pDD = pDevice->getCurrentDither ();

   pEnum = DeviceDither::getEnumeration ();

   cout << "*DefaultDither: " << DeviceDither::IDToName (pDD->getID ()) << endl;

   while (pEnum->hasMoreElements ())
   {
      char *pszID = (char *)pEnum->nextElement ();

      cout << "*Dither "
           << pszID
           << "/"
           << StringResource::getString (StringResource::STRINGGROUP_DITHERS, pszID)
           << ":\t\"<</OutputType("
           // @TBD - should this be our settable name?
           << StringResource::getString (StringResource::STRINGGROUP_DITHERS, pszID)
           << ")>>setpagedevice\""
           << endl;
   }

   cout << "*CloseUI: *Dither" << endl;

   delete pEnum;

   cout << "*OpenUI *Resolution: PickOne" << endl;
   cout << "*OrderDependency: 20 AnySetup *Resolution" << endl;

   DeviceResolution *pDR = pDevice->getCurrentResolution ();

   cout << "*DefaultResolution: " << DeviceResolution::IDToName (pDR->getID ()) << endl;

   pEnum = pDR->getEnumeration ();

   while (pEnum->hasMoreElements ())
   {
      pDR = (DeviceResolution *)pEnum->nextElement ();

      cout << "*Resolution "
           << DeviceResolution::IDToName (pDR->getID ())
           << "/"
           << pDR->getName ()
           << ":\t\"<</HWResolution["
           << pDR->getXRes ()
           << " "
           << pDR->getYRes ()
           << "]/cupsCompression "
           << pDR->getID ()        // @TBD - correct?
           << ">>setpagedevice\""
           << endl;

      delete pDR;
   }

   cout << "*CloseUI: *Resolution" << endl;

   delete pEnum;

   cout << "*DefaultFont: Courier" << endl;
   cout << "*Font AvantGarde-Book: Standard \"(001.006S)\" Standard ROM" << endl;
   cout << "*Font AvantGarde-BookOblique: Standard \"(001.006S)\" Standard ROM" << endl;
   cout << "*Font AvantGarde-Demi: Standard \"(001.007S)\" Standard ROM" << endl;
   cout << "*Font AvantGarde-DemiOblique: Standard \"(001.007S)\" Standard ROM" << endl;
   cout << "*Font Bookman-Demi: Standard \"(001.004S)\" Standard ROM" << endl;
   cout << "*Font Bookman-DemiItalic: Standard \"(001.004S)\" Standard ROM" << endl;
   cout << "*Font Bookman-Light: Standard \"(001.004S)\" Standard ROM" << endl;
   cout << "*Font Bookman-LightItalic: Standard \"(001.004S)\" Standard ROM" << endl;
   cout << "*Font Courier: Standard \"(002.004S)\" Standard ROM" << endl;
   cout << "*Font Courier-Bold: Standard \"(002.004S)\" Standard ROM" << endl;
   cout << "*Font Courier-BoldOblique: Standard \"(002.004S)\" Standard ROM" << endl;
   cout << "*Font Courier-Oblique: Standard \"(002.004S)\" Standard ROM" << endl;
   cout << "*Font Helvetica: Standard \"(001.006S)\" Standard ROM" << endl;
   cout << "*Font Helvetica-Bold: Standard \"(001.007S)\" Standard ROM" << endl;
   cout << "*Font Helvetica-BoldOblique: Standard \"(001.007S)\" Standard ROM" << endl;
   cout << "*Font Helvetica-Narrow: Standard \"(001.006S)\" Standard ROM" << endl;
   cout << "*Font Helvetica-Narrow-Bold: Standard \"(001.007S)\" Standard ROM" << endl;
   cout << "*Font Helvetica-Narrow-BoldOblique: Standard \"(001.007S)\" Standard ROM" << endl;
   cout << "*Font Helvetica-Narrow-Oblique: Standard \"(001.006S)\" Standard ROM" << endl;
   cout << "*Font Helvetica-Oblique: Standard \"(001.006S)\" Standard ROM" << endl;
   cout << "*Font NewCenturySchlbk-Bold: Standard \"(001.009S)\" Standard ROM" << endl;
   cout << "*Font NewCenturySchlbk-BoldItalic: Standard \"(001.007S)\" Standard ROM" << endl;
   cout << "*Font NewCenturySchlbk-Italic: Standard \"(001.006S)\" Standard ROM" << endl;
   cout << "*Font NewCenturySchlbk-Roman: Standard \"(001.007S)\" Standard ROM" << endl;
   cout << "*Font Palatino-Bold: Standard \"(001.005S)\" Standard ROM" << endl;
   cout << "*Font Palatino-BoldItalic: Standard \"(001.005S)\" Standard ROM" << endl;
   cout << "*Font Palatino-Italic: Standard \"(001.005S)\" Standard ROM" << endl;
   cout << "*Font Palatino-Roman: Standard \"(001.005S)\" Standard ROM" << endl;
   cout << "*Font Symbol: Special \"(001.007S)\" Special ROM" << endl;
   cout << "*Font Times-Bold: Standard \"(001.007S)\" Standard ROM" << endl;
   cout << "*Font Times-BoldItalic: Standard \"(001.009S)\" Standard ROM" << endl;
   cout << "*Font Times-Italic: Standard \"(001.007S)\" Standard ROM" << endl;
   cout << "*Font Times-Roman: Standard \"(001.007S)\" Standard ROM" << endl;
   cout << "*Font ZapfChancery-MediumItalic: Standard \"(001.007S)\" Standard ROM" << endl;
   cout << "*Font ZapfDingbats: Special \"(001.004S)\" Standard ROM" << endl;

   cout << "*%%End of " << pszDeviceName << ".ppd" << endl;

   delete pDevice;

   return 0;
}
