/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <fstream.h>

#include "Device.hpp"
#include "Omni.hpp"

#include <sys/stat.h>
#include <sys/types.h>
#include <glib.h>
#include <gmodule.h>

#include "omni2foo.h"

string *
getDeviceName (Device *pDevice, omni2foo* db)
{
   ostrstream oss;
   oss << pDevice->getShortName () << ends;
   
   // check the db
   string id=db->getFooIDFromString (oss.str ());
   // if there is no match, the string that is returned is empty
   if(id.length () > 0)
   {
      // found a match, return the fooid string
      return new string (id);
   }
   else
   {
      // no match, return orginal device name
      // Mangle the device name to be Manu-Make
      // First space needs to be a -, and then all spaces after that need to be a _
      // Ex:  Canon-BJC-2000, Epson-Stylus_Color_740
      //
      string device=(pDevice->getDeviceName ());
      // Find the first space, ie, the gap between the make and model
      // and replace the " " with a "-"
      string::size_type pos = device.find_first_of(" ",0);
      device.replace (pos,1,"-");

      // Now do a substring replace, change all spaces to _
      for(unsigned int i=0; i < device.size (); i++)
      {
         if( device[i]==' ' ) device[i]='_';
      }
         
      return new string (device );
   }
}

int main (int argc, char *argv[])
{
   string basedir="";
   string foomaticDB="bin/newDB";

   /*
   // Command Line parsing.  2 options.  
   // -b : base dir, the location where the foomatic-db dir will be made
   // -d : the foomatic id db.  it is a space separated file, specify where that file is, or use the default
   for (int i = 1; i < argc; i ++)
   {
      if (argv[i][0] == '-')
      {
         switch (argv[i][1])
         {
            case D:
            case d:
                foomaticDB=
    */

   
   typedef struct _OmniDeviceHandle {
      GModule *hmodDevice;
      Device  *pDevice;
      string  *pstringDeviceName;
   } OMNIHANDLE, *POMNIHANDLE;

   POMNIHANDLE  pohDevices = 0;
   ofstream    *pofstream;

   if (!g_module_supported ())
   {
      cerr << "This program needs glib's module routines!" << endl;
      return 1;
   }

   // Now make the omni2foo DB
   omni2foo::omni2foo DB(foomaticDB);

   mode_t mode = S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP;

   mkdir ("foomatic-db", mode);
   mkdir ("foomatic-db/db", mode);
   mkdir ("foomatic-db/db/source", mode);
   mkdir ("foomatic-db/db/source/driver", mode);
   mkdir ("foomatic-db/db/source/printer", mode);
   mkdir ("foomatic-db/db/source/opt", mode);

   cout << "Counting devices..." << endl;

   Enumeration *pEnum          = Omni::listDevices ();
   int          iNumDevices    = 0;
   int          iCurrentDevice = 0;

   while (pEnum->hasMoreElements ())
   {
      pEnum->nextElement ();

      iNumDevices++;
   }

   delete pEnum;

   cout << "There are " << iNumDevices << " devices." << endl;
   cout << "Initializing devices...";
   cout.flush ();

   pohDevices = new OMNIHANDLE [iNumDevices];
   pEnum      = Omni::listDevices ();

   while (pEnum->hasMoreElements ())
   {
      char         *pszLibName   = (char *)pEnum->nextElement ();
      PFNNEWDEVICE  pfnNewDevice = 0;

      pohDevices[iCurrentDevice].hmodDevice = g_module_open (pszLibName, (GModuleFlags)0);

      if (pohDevices[iCurrentDevice].hmodDevice)
      {
         g_module_symbol (pohDevices[iCurrentDevice].hmodDevice, "newDevice__Fb", (gpointer *)&pfnNewDevice);

         if (pfnNewDevice)
         {
            Device *pDevice = 0;

            pDevice                                      = pfnNewDevice (true);
            pohDevices[iCurrentDevice].pDevice           = pDevice;
            pohDevices[iCurrentDevice].pstringDeviceName = getDeviceName (pDevice,&DB);
         }
      }
      else
      {
         pohDevices[iCurrentDevice].pDevice           = 0;
         pohDevices[iCurrentDevice].pstringDeviceName = 0;
      }

      iCurrentDevice++;
   }

   delete pEnum;

   cout << "done." << endl;
   

   for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
   {
      Device *pDevice           = pohDevices[iCurrentDevice].pDevice;
      string *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;
      char    achFilename[128];

      sprintf (achFilename, "foomatic-db/db/source/printer/%s.xml", pstringDeviceName->c_str ());
      ofstream ofstreamPrinter (achFilename);
      pofstream = &ofstreamPrinter;

      sprintf (achFilename, "printer/%s.xml", pstringDeviceName->c_str ());
      cout << "Generating " << achFilename << endl;

      *pofstream << "<printer id=\"" << achFilename << "\">" << endl;
      *pofstream << "  <make>" << pDevice->getDriverName () << "</make>" << endl;
      *pofstream << "  <model>" << pDevice->getShortName () << "</model>" << endl;
//////*pofstream << "  <model>" << pDevice->getDeviceName () << "</model>" << endl;
      *pofstream << "  <mechanism>" << endl;
      *pofstream << "    <inkjet/>" << endl;
      *pofstream << "    <color/>" << endl;
      *pofstream << "    <resolution>" << endl;
      *pofstream << "      <dpi>" << endl;
      //  Added by raharper@us.ibm.com, get the resolution for each device
      *pofstream << "        <x>" << pDevice->getCurrentResolution()->getXRes() << "</x>" << endl;
      *pofstream << "        <y>" << pDevice->getCurrentResolution()->getYRes() << "</y>" << endl;
      *pofstream << "      </dpi>" << endl;
      *pofstream << "    </resolution>" << endl;
      *pofstream << "    <consumables>" << endl;
      *pofstream << "      <comments>" << endl;
      //  Added by raharper@us.ibm.com, changed Comment to something device specific
      *pofstream << "        <en>" << pDevice->getShortName () << "</en>" << endl;
      *pofstream << "      </comments>" << endl;
      *pofstream << "    <!--one or more \"partno\" elements.--></consumables>" << endl;
      *pofstream << "  </mechanism>" << endl;
      //  Added by raharper@us.ibm.com, changed URL
      *pofstream << "  <url>http://www.sourceforge.net/projects/omniprint</url>" << endl;
      *pofstream << "  <lang>" << endl;
      *pofstream << "    <pcl level=\"3\"/><!--no pjl-->" << endl;
      *pofstream << "    <text>" << endl;
      *pofstream << "      <charset>us-ascii</charset>" << endl;
      *pofstream << "    </text>" << endl;
      *pofstream << "  </lang>" << endl;
      *pofstream << "  <autodetect>" << endl;
      *pofstream << "    <!--no known parport probe information-->" << endl;
      *pofstream << "  </autodetect>" << endl;
      *pofstream << "  <functionality>A</functionality><!--unknown preferred \"driver\"-->" << endl;
      *pofstream << "  <unverified/><!--no \"contrib_url\"-->" << endl;
      *pofstream << "  <comments>" << endl;
      *pofstream << "    <en>" << endl;
      // Added by raharper@us.ibm.com, changing to simple comment about omni
      *pofstream << "     http://www.sourceforge.net/projects/omniprint" << endl;
      *pofstream << "" << endl;
      *pofstream << "    </en>" << endl;
      *pofstream << "  </comments>" << endl;
      *pofstream << "</printer>" << endl;
   }

   /**
    * Create Omni
    *
    * list of device names
    */
   ofstream ofstreamOmni ("foomatic-db/db/source/driver/omni.xml");
   pofstream = &ofstreamOmni;

   cout << "Generating omni.xml" << endl;

   *pofstream << "<driver id=\"driver/omni\">" << endl;
   *pofstream << " <name>omni-print</name>" << endl;
   *pofstream << " <url>http://www.ibm.com/linux/ltc/projects/omni/</url>" << endl;
   *pofstream << " <execution>" << endl;
   *pofstream << "  <ghostscript />" << endl;
   // Edited by raharper@us.ibm.com, changed the prototype to what omni wants it to look like
   *pofstream << "  <prototype>gs -q -dSAFER -dBATCH -dNOPAUSE -sDEVICE=omni -sDeviceName=%A%Z -sOutputFile=- -</prototype>" << endl; // @TBD
   *pofstream << " </execution>" << endl;
   *pofstream << " <comments>" << endl;
   *pofstream << "   <en>" << endl;
   *pofstream << "   @TBD" << endl;
   *pofstream << "   </en>" << endl;
   *pofstream << " </comments>" << endl;
   *pofstream << "  <printers>" << endl;

   for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
   {
      string *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

      *pofstream << "  <printer><id>printer/" << *pstringDeviceName << "</id></printer>" << endl;
   }

   *pofstream << "  </printers>" << endl;
   *pofstream << "</driver>" << endl;

   /**
    * Create Print Model
    *
    * list of device names
    */
   ofstream ofstreamPrintModel ("foomatic-db/db/source/opt/omni-print-model.xml");
   pofstream = &ofstreamPrintModel;

   cout << "Generating omni-print-model.xml" << endl;

   *pofstream << "<option type=\"enum\" id=\"opt/omni-print-model\">" << endl;
   *pofstream << "  <!-- A multilingual <comments> block can appear here, too;" << endl;
   *pofstream << "       it should be treated as documentation for the user. -->" << endl;
   *pofstream << "  <arg_longname>" << endl;
   *pofstream << "   <en>Printer Model</en>" << endl;
   *pofstream << "  </arg_longname>" << endl;
   *pofstream << "  <arg_shortname>" << endl;
   *pofstream << "   <en>Model</en><!-- backends only know <en> shortnames! -->" << endl;
   *pofstream << "  </arg_shortname>" << endl;
   *pofstream << "  <arg_execution>" << endl;
   *pofstream << "   <arg_order>10</arg_order>" << endl;
   *pofstream << "   <arg_spot>A</arg_spot>" << endl;
   *pofstream << "   <arg_required />" << endl;
   *pofstream << "   <arg_substitution />" << endl;
   *pofstream << "   <arg_proto>%s -sproperties\" </arg_proto>" << endl;
   *pofstream << "  </arg_execution>" << endl;
   *pofstream << "" << endl;
   *pofstream << "  <constraints>" << endl;

   for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
   {
      string *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

      *pofstream << "    <constraint sense='true'>" << endl;
      *pofstream << "      <driver>omni</driver>" << endl;
      *pofstream << "      <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
      *pofstream << "      <arg_defval>" << *pstringDeviceName << "</arg_defval>" << endl; // @TBD
      *pofstream << "    </constraint>" << endl;
   }

   *pofstream << "  </constraints>" << endl;
   *pofstream << endl;
   *pofstream << "  <enum_vals>" << endl;

   for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
   {
      Device *pDevice           = pohDevices[iCurrentDevice].pDevice;
      string *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

      //raharper@us.ibm.com took out the ev/omni-
      *pofstream << "    <enum_val id='" << *pstringDeviceName << "'>" << endl;
      *pofstream << "      <ev_longname><en>" << *pstringDeviceName << "</en></ev_longname>" << endl;
      *pofstream << "      <ev_shortname><en>" << pDevice->getShortName () << "</en></ev_shortname>" << endl;
      *pofstream << "      <ev_driverval>" << pDevice->getShortName () << "</ev_driverval>" << endl;
      *pofstream << "      <constraints>" << endl;
      *pofstream << "        <!-- Assume the option doesn't apply... -->" << endl;
      *pofstream << "        <constraint sense='false'>" << endl;
      *pofstream << "          <driver>omni</driver>" << endl;
      *pofstream << "        </constraint>" << endl;
      *pofstream << "        <!-- ...except to these: -->" << endl;
      *pofstream << "        <constraint sense='true'>" << endl;
      *pofstream << "          <driver>omni</driver>" << endl;
      *pofstream << "          <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
      *pofstream << "        </constraint>" << endl;
      *pofstream << "      </constraints>" << endl;
      *pofstream << "    </enum_val>" << endl;
   }

   *pofstream << "  </enum_vals>" << endl;
   *pofstream << "</option>" << endl;

   /**
    * Create Paper Sizes(Forms)
    *
    * list of forms
    */
   // raharper@us.ibm.com, changed filename to form
   ofstream ofstreamForm ("foomatic-db/db/source/opt/omni-form.xml");
   pofstream = &ofstreamForm;

   cout << "Generating omni-form.xml" << endl;

   *pofstream << "<option type=\"enum\" id=\"opt/omni-form\">" << endl;
   *pofstream << "  <arg_longname>" << endl;
   // Edited by raharper@us.ibm.com, changing the output to match omni driver options
   *pofstream << "   <en>Form</en>" << endl;
   *pofstream << "  </arg_longname>" << endl;
   *pofstream << "  <arg_shortname>" << endl;
   // Edited by raharper@us.ibm.com, changing the output to match omni driver options
   *pofstream << "   <en>PageSize</en><!-- backends only know <en> shortnames! -->" << endl;
   *pofstream << "  </arg_shortname>" << endl;
   *pofstream << "  <arg_execution>" << endl;
   *pofstream << "   <arg_order>100</arg_order>" << endl;
   *pofstream << "   <arg_spot>A</arg_spot>" << endl;
   *pofstream << "   <arg_required />" << endl;
   // raharper@us.ibm.com, changed the arg_xxxx type to subsitutiuon to get the style=C in the perl dat structure generator, db->getdat()
   *pofstream << "   <arg_substitution />" << endl;
   // Edited by raharper@us.ibm.com, changing the prototype to what omni uses for commandline options
   *pofstream << "   <arg_proto> form=%s</arg_proto>" << endl;
   *pofstream << "  </arg_execution>" << endl;
   *pofstream << endl;

   *pofstream << "  <constraints>" << endl;

   for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
   {
      Device     *pDevice           = pohDevices[iCurrentDevice].pDevice;
      DeviceForm *pDF               = pDevice->getCurrentForm ();
      string     *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

      *pofstream << "    <constraint sense='true'>" << endl;
      *pofstream << "      <driver>omni</driver>" << endl;
      *pofstream << "      <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
      // raharper@us.ibm.com, changing the arg_defval output
      *pofstream << "      <arg_defval>" << DeviceForm::IDToName (pDF->getID ()) << "</arg_defval>" << endl;
      *pofstream << "    </constraint>" << endl;
   }

   *pofstream << "  </constraints>" << endl;
   *pofstream << endl;
   *pofstream << "  <enum_vals>" << endl;

   for (int iForm = DeviceForm::FORM_NONE; iForm < DeviceForm::COUNT; iForm++)
   {
      bool fFirstTime = true;

      for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
      {
         Device     *pDevice           = pohDevices[iCurrentDevice].pDevice;
         DeviceForm *pDF               = pDevice->getCurrentForm ();
         string     *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

         if (pDF->isFormSupported (iForm))
         {
            if (fFirstTime)
            {
               fFirstTime = false;

               //raharper@us.ibm.com took out the ev/omni-
               *pofstream << "    <enum_val id='" << DeviceForm::IDToName (iForm) << "'>" << endl;
               *pofstream << "      <ev_longname><en>" << DeviceForm::getName (iForm) << "</en></ev_longname>" << endl;
               *pofstream << "      <ev_shortname><en>" << DeviceForm::IDToName (iForm) << "</en></ev_shortname>" << endl;
               *pofstream << "      <ev_driverval>" << DeviceForm::IDToName (iForm) << "</ev_driverval>" << endl;
               *pofstream << "      <constraints>" << endl;
               *pofstream << "        <!-- Assume the option doesn't apply... -->" << endl;
               *pofstream << "        <constraint sense='false'>" << endl;
               *pofstream << "          <driver>omni</driver>" << endl;
               *pofstream << "        </constraint>" << endl;
            }

            *pofstream << "        <constraint sense='true'>" << endl;
            *pofstream << "          <driver>omni</driver>" << endl;
            *pofstream << "          <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
            *pofstream << "        </constraint>" << endl;
         }
      }

      if (!fFirstTime)
      {
         *pofstream << "      </constraints>" << endl;
         *pofstream << "    </enum_val>" << endl;
      }
   }

   *pofstream << "  </enum_vals>" << endl;
   *pofstream << "</option>" << endl;

   /**
    * Create Input Slots
    *
    * list of trays
    */
   // raharper@us.ibm.com, changed file name to tray
   ofstream ofstreamTray ("foomatic-db/db/source/opt/omni-tray.xml");
   pofstream = &ofstreamTray;

   cout << "Generating omni-tray.xml" << endl;

   *pofstream << "<option type=\"enum\" id=\"opt/omni-tray\">" << endl;
   *pofstream << "  <!-- A multilingual <comments> block can appear here, too;" << endl;
   *pofstream << "       it should be treated as documentation for the user. -->" << endl;
   *pofstream << "  <arg_longname>" << endl;
   // Edited by raharper@us.ibm.com, changing the output to match omni driver options
   *pofstream << "   <en>Tray</en>" << endl;
   *pofstream << "  </arg_longname>" << endl;
   *pofstream << "  <arg_shortname>" << endl;
   // Edited by raharper@us.ibm.com, changing the output to match omni driver options
   *pofstream << "   <en>InputSlot</en><!-- backends only know <en> shortnames! -->" << endl;
   *pofstream << "  </arg_shortname>" << endl;
   *pofstream << "  <arg_execution>" << endl;
   *pofstream << "   <arg_order>100</arg_order>" << endl;
   *pofstream << "   <arg_spot>A</arg_spot>" << endl;
   *pofstream << "   <arg_substitution />" << endl;
   // Edited by raharper@us.ibm.com, changing the output to match omni driver options
   *pofstream << "   <arg_proto> tray=%s</arg_proto>" << endl;
   *pofstream << "  </arg_execution>" << endl;
   *pofstream << endl;

   *pofstream << "  <constraints>" << endl;

   for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
   {
      Device     *pDevice           = pohDevices[iCurrentDevice].pDevice;
      DeviceTray *pDT               = pDevice->getCurrentTray ();
      string     *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

      *pofstream << "    <constraint sense='true'>" << endl;
      *pofstream << "      <driver>omni</driver>" << endl;
      *pofstream << "      <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
      // raharper@us.ibm.com, changing what the defval output looks like
      *pofstream << "      <arg_defval>" << DeviceTray::IDToName (pDT->getID ()) << "</arg_defval>" << endl;
      *pofstream << "    </constraint>" << endl;
   }

   *pofstream << "  </constraints>" << endl;
   *pofstream << endl;
   *pofstream << "  <enum_vals>" << endl;

   for (int iTray = DeviceTray::TRAY_NONE; iTray < DeviceTray::COUNT; iTray++)
   {
      bool fFirstTime = true;

      for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
      {
         Device     *pDevice           = pohDevices[iCurrentDevice].pDevice;
         DeviceTray *pDT               = pDevice->getCurrentTray ();
         string     *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

         if (pDT->isTraySupported (iTray))
         {
            if (fFirstTime)
            {
               fFirstTime = false;

               //raharper@us.ibm.com took out the ev/omni-
               *pofstream << "    <enum_val id='" << DeviceTray::IDToName (iTray) << "'>" << endl;
               *pofstream << "      <ev_longname><en>" << DeviceTray::getName (iTray) << "</en></ev_longname>" << endl;
               *pofstream << "      <ev_shortname><en>" << DeviceTray::IDToName (iTray) << "</en></ev_shortname>" << endl;
               *pofstream << "      <ev_driverval>" << DeviceTray::IDToName (iTray) << "</ev_driverval>" << endl;
               *pofstream << "      <constraints>" << endl;
               *pofstream << "        <!-- Assume the option doesn't apply... -->" << endl;
               *pofstream << "        <constraint sense='false'>" << endl;
               *pofstream << "          <driver>omni</driver>" << endl;
               *pofstream << "        </constraint>" << endl;
            }

            *pofstream << "        <constraint sense='true'>" << endl;
            *pofstream << "          <driver>omni</driver>" << endl;
            *pofstream << "          <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
            *pofstream << "        </constraint>" << endl;
         }
      }

      if (!fFirstTime)
      {
         *pofstream << "      </constraints>" << endl;
         *pofstream << "    </enum_val>" << endl;
      }
   }

   *pofstream << "  </enum_vals>" << endl;
   *pofstream << "</option>" << endl;

   /**
    * Create MediaTypes
    *
    * list of medias
    */
   // raharper@us.ibm.com, Added Media section
   ofstream ofstreamInputSlots ("foomatic-db/db/source/opt/omni-media.xml");
   pofstream = &ofstreamInputSlots;

   cout << "Generating omni-media.xml" << endl;

   *pofstream << "<option type=\"enum\" id=\"opt/omni-media\">" << endl;
   *pofstream << "  <!-- A multilingual <comments> block can appear here, too;" << endl;
   *pofstream << "       it should be treated as documentation for the user. -->" << endl;
   *pofstream << "  <arg_longname>" << endl;
   // Edited by raharper@us.ibm.com, changing the output to match omni driver options
   *pofstream << "   <en>Media</en>" << endl;
   *pofstream << "  </arg_longname>" << endl;
   *pofstream << "  <arg_shortname>" << endl;
   // Edited by raharper@us.ibm.com, changing the output to match omni driver options
   *pofstream << "   <en>MediaType</en><!-- backends only know <en> shortnames! -->" << endl;
   *pofstream << "  </arg_shortname>" << endl;
   *pofstream << "  <arg_execution>" << endl;
   *pofstream << "   <arg_order>100</arg_order>" << endl;
   *pofstream << "   <arg_spot>A</arg_spot>" << endl;
   *pofstream << "   <arg_substitution />" << endl;
   // Edited by raharper@us.ibm.com, changing the output to match omni driver options
   *pofstream << "   <arg_proto> media=%s</arg_proto>" << endl;
   *pofstream << "  </arg_execution>" << endl;
   *pofstream << endl;

   *pofstream << "  <constraints>" << endl;

   for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
   {
      Device      *pDevice           = pohDevices[iCurrentDevice].pDevice;
      DeviceMedia *pDT               = pDevice->getCurrentMedia ();
      string      *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

      *pofstream << "    <constraint sense='true'>" << endl;
      *pofstream << "      <driver>omni</driver>" << endl;
      *pofstream << "      <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
      // raharper@us.ibm.com, changing what the defval output looks like
      *pofstream << "      <arg_defval>" << DeviceMedia::IDToName (pDT->getID ()) << "</arg_defval>" << endl;
      *pofstream << "    </constraint>" << endl;
   }

   *pofstream << "  </constraints>" << endl;
   *pofstream << endl;
   *pofstream << "  <enum_vals>" << endl;

   for (int iMedia = DeviceMedia::MEDIA_NONE; iMedia < DeviceMedia::COUNT; iMedia++)
   {
      bool fFirstTime = true;

      for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
      {
         Device     *pDevice           = pohDevices[iCurrentDevice].pDevice;
         DeviceMedia *pDT               = pDevice->getCurrentMedia ();
         string     *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

         if (pDT->isMediaSupported (iMedia))
         {
            if (fFirstTime)
            {
               fFirstTime = false;
               //raharper@us.ibm.com took out the ev/omni-
               *pofstream << "    <enum_val id='" << DeviceMedia::IDToName (iMedia) << "'>" << endl;
               *pofstream << "      <ev_longname><en>" << DeviceMedia::getName (iMedia) << "</en></ev_longname>" << endl;
               *pofstream << "      <ev_shortname><en>" << DeviceMedia::IDToName (iMedia) << "</en></ev_shortname>" << endl;
               *pofstream << "      <ev_driverval>" << DeviceMedia::IDToName (iMedia) << "</ev_driverval>" << endl;
               *pofstream << "      <constraints>" << endl;
               *pofstream << "        <!-- Assume the option doesn't apply... -->" << endl;
               *pofstream << "        <constraint sense='false'>" << endl;
               *pofstream << "          <driver>omni</driver>" << endl;
               *pofstream << "        </constraint>" << endl;
            }

            *pofstream << "        <constraint sense='true'>" << endl;
            *pofstream << "          <driver>omni</driver>" << endl;
            *pofstream << "          <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
            *pofstream << "        </constraint>" << endl;
         }
      }

      if (!fFirstTime)
      {
         *pofstream << "      </constraints>" << endl;
         *pofstream << "    </enum_val>" << endl;
      }
   }

   *pofstream << "  </enum_vals>" << endl;
   *pofstream << "</option>" << endl;

   /**
    * Create Resolution
    *
    * list of resolutions
    */
   ofstream ofstreamResolutions ("foomatic-db/db/source/opt/omni-resolution.xml");
   pofstream = &ofstreamResolutions;

   cout << "Generating omni-resolution.xml" << endl;
   // raharper@us.ibm.com, changed GSResolution to Resolution
   *pofstream << "<option type=\"enum\" id=\"opt/omni-resolution\">" << endl;
   *pofstream << "  <!-- A multilingual <comments> block can appear here, too;" << endl;
   *pofstream << "       it should be treated as documentation for the user. -->" << endl;
   *pofstream << "  <arg_longname>" << endl;
   *pofstream << "   <en>Resolution</en>" << endl;
   *pofstream << "  </arg_longname>" << endl;
   *pofstream << "  <arg_shortname>" << endl;
   *pofstream << "   <en>Resolution</en><!-- backends only know <en> shortnames! -->" << endl;
   *pofstream << "  </arg_shortname>" << endl;
   *pofstream << "  <arg_execution>" << endl;
   *pofstream << "   <arg_order>100</arg_order>" << endl;
   *pofstream << "   <arg_spot>A</arg_spot>" << endl;
   *pofstream << "   <arg_substitution />" << endl;
   // edited by raharper@us.ibm.com, changed output to match omni driver options
   *pofstream << "   <arg_proto> resolution=%s</arg_proto>" << endl;
   *pofstream << "  </arg_execution>" << endl;
   *pofstream << endl;

   *pofstream << "  <constraints>" << endl;

   for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
   {
      Device           *pDevice           = pohDevices[iCurrentDevice].pDevice;
      DeviceResolution *pDR               = pDevice->getCurrentResolution ();
      string           *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

      *pofstream << "    <constraint sense='true'>" << endl;
      *pofstream << "      <driver>omni</driver>" << endl;
      *pofstream << "      <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
      // raharper@us.ibm.com, changed the defval output
      *pofstream << "      <arg_defval>" << DeviceResolution::IDToName (pDR->getID ()) << "</arg_defval>" << endl;
      *pofstream << "    </constraint>" << endl;
   }

   *pofstream << "  </constraints>" << endl;
   *pofstream << endl;
   *pofstream << "  <enum_vals>" << endl;

   for (int iResolution = DeviceResolution::RESOLUTION_NONE; iResolution < DeviceResolution::COUNT; iResolution++)
   {
      bool fFirstTime = true;

      for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
      {
         Device           *pDevice           = pohDevices[iCurrentDevice].pDevice;
         DeviceResolution *pDR               = pDevice->getCurrentResolution ();
         string           *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

         if (pDR->isResolutionSupported (iResolution))
         {
            if (fFirstTime)
            {
               fFirstTime = false;

               //raharper@us.ibm.com took out the ev/omni-
               *pofstream << "    <enum_val id='" << DeviceResolution::IDToName (iResolution) << "'>" << endl;
               *pofstream << "      <ev_longname><en>" << DeviceResolution::getName (iResolution) << "</en></ev_longname>" << endl;
               *pofstream << "      <ev_shortname><en>" << DeviceResolution::IDToName (iResolution) << "</en></ev_shortname>" << endl;
               *pofstream << "      <ev_driverval>" << DeviceResolution::IDToName (iResolution) << "</ev_driverval>" << endl;
               *pofstream << "      <constraints>" << endl;
               *pofstream << "        <!-- Assume the option doesn't apply... -->" << endl;
               *pofstream << "        <constraint sense='false'>" << endl;
               *pofstream << "          <driver>omni</driver>" << endl;
               *pofstream << "        </constraint>" << endl;
            }

            *pofstream << "        <constraint sense='true'>" << endl;
            *pofstream << "          <driver>omni</driver>" << endl;
            *pofstream << "          <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
            *pofstream << "        </constraint>" << endl;
         }
      }

      if (!fFirstTime)
      {
         *pofstream << "      </constraints>" << endl;
         *pofstream << "    </enum_val>" << endl;
      }
   }

   *pofstream << "  </enum_vals>" << endl;
   *pofstream << "</option>" << endl;

   /**
    * Create Ink Type
    *
    * list of printmodes
    */
   // raharper@us.ibm.com changed filename to printmode
   ofstream ofstreamPrintModes ("foomatic-db/db/source/opt/omni-printmode.xml");
   pofstream = &ofstreamPrintModes;

   cout << "Generating omni-inktype.xml" << endl;

   *pofstream << "<option type=\"enum\" id=\"opt/omni-inktype\">" << endl;
   *pofstream << "  <!-- A multilingual <comments> block can appear here, too;" << endl;
   *pofstream << "       it should be treated as documentation for the user. -->" << endl;
   *pofstream << "  <arg_longname>" << endl;
   // raharper@us.ibm.com, changed longname
   *pofstream << "   <en>Print Mode</en>" << endl;
   *pofstream << "  </arg_longname>" << endl;
   *pofstream << "  <arg_shortname>" << endl;
   *pofstream << "   <en>PrintMode</en><!-- backends only know <en> shortnames! -->" << endl;
   *pofstream << "  </arg_shortname>" << endl;
   *pofstream << "  <arg_execution>" << endl;
   *pofstream << "   <arg_order>100</arg_order>" << endl;
   *pofstream << "   <arg_spot>A</arg_spot>" << endl;
   *pofstream << "   <arg_substitution />" << endl;
   // raharper@us.ibm.com, changed prototype
   *pofstream << "   <arg_proto> printmode=%s</arg_proto>" << endl;
   *pofstream << "  </arg_execution>" << endl;
   *pofstream << endl;

   *pofstream << "  <constraints>" << endl;

   for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
   {
      Device          *pDevice           = pohDevices[iCurrentDevice].pDevice;
      DevicePrintMode *pDR               = pDevice->getCurrentPrintMode ();
      string          *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

      *pofstream << "    <constraint sense='true'>" << endl;
      *pofstream << "      <driver>omni</driver>" << endl;
      *pofstream << "      <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
      // raharper@us.ibm.com, changed defval
      *pofstream << "      <arg_defval>" << DevicePrintMode::IDToName (pDR->getID ()) << "</arg_defval>" << endl;
      *pofstream << "    </constraint>" << endl;
   }

   *pofstream << "  </constraints>" << endl;
   *pofstream << endl;
   *pofstream << "  <enum_vals>" << endl;

   for (int iPrintMode = DevicePrintMode::PRINT_MODE_NONE; iPrintMode < DevicePrintMode::COUNT; iPrintMode++)
   {
      bool fFirstTime = true;

      for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
      {
         Device          *pDevice           = pohDevices[iCurrentDevice].pDevice;
         DevicePrintMode *pDPM              = pDevice->getCurrentPrintMode ();
         string          *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

         if (pDPM->isPrintModeSupported (iPrintMode))
         {
            if (fFirstTime)
            {
               fFirstTime = false;

               //raharper@us.ibm.com took out the ev/omni-
               *pofstream << "    <enum_val id='" << DevicePrintMode::IDToName (iPrintMode) << "'>" << endl;
               *pofstream << "      <ev_longname><en>" << DevicePrintMode::getName (iPrintMode) << "</en></ev_longname>" << endl;
               *pofstream << "      <ev_shortname><en>" << DevicePrintMode::IDToName (iPrintMode) << "</en></ev_shortname>" << endl;
               *pofstream << "      <ev_driverval>" << DevicePrintMode::IDToName (iPrintMode) << "</ev_driverval>" << endl;
               *pofstream << "      <constraints>" << endl;
               *pofstream << "        <!-- Assume the option doesn't apply... -->" << endl;
               *pofstream << "        <constraint sense='false'>" << endl;
               *pofstream << "          <driver>omni</driver>" << endl;
               *pofstream << "        </constraint>" << endl;
            }

            *pofstream << "        <constraint sense='true'>" << endl;
            *pofstream << "          <driver>omni</driver>" << endl;
            *pofstream << "          <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
            *pofstream << "        </constraint>" << endl;
         }
      }

      if (!fFirstTime)
      {
         *pofstream << "      </constraints>" << endl;
         *pofstream << "    </enum_val>" << endl;
      }
   }

   *pofstream << "  </enum_vals>" << endl;
   *pofstream << "</option>" << endl;

   /**
    * Create Image Type
    *
    * list of dithers
    */
   ofstream ofstreamDithers ("foomatic-db/db/source/opt/omni-dither.xml");
   pofstream = &ofstreamDithers;

   cout << "Generating omni-dither.xml" << endl;

   *pofstream << "<option type=\"enum\" id=\"opt/omni-dither\">" << endl;
   *pofstream << "  <!-- A multilingual <comments> block can appear here, too;" << endl;
   *pofstream << "       it should be treated as documentation for the user. -->" << endl;
   *pofstream << "  <arg_longname>" << endl;
   *pofstream << "   <en>Dither Algorithm</en>" << endl;
   *pofstream << "  </arg_longname>" << endl;
   *pofstream << "  <arg_shortname>" << endl;
   *pofstream << "   <en>Dither</en><!-- backends only know <en> shortnames! -->" << endl;
   *pofstream << "  </arg_shortname>" << endl;
   *pofstream << "  <arg_execution>" << endl;
   *pofstream << "   <arg_order>100</arg_order>" << endl;
   *pofstream << "   <arg_spot>A</arg_spot>" << endl;
   *pofstream << "   <arg_substitution />" << endl;
   // raharper@us.ibm.com, changed prototype
   *pofstream << "   <arg_proto> dither=%s</arg_proto>" << endl;
   *pofstream << "  </arg_execution>" << endl;
   *pofstream << endl;

   *pofstream << "  <constraints>" << endl;

   for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
   {
      Device *pDevice           = pohDevices[iCurrentDevice].pDevice;
      string *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

      *pofstream << "    <constraint sense='true'>" << endl;
      *pofstream << "      <driver>omni</driver>" << endl;
      *pofstream << "      <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
      // raharper@us.ibm.com, changed defval
      *pofstream << "      <arg_defval>" << pDevice->getCurrentDitherID () << "</arg_defval>" << endl;
      *pofstream << "    </constraint>" << endl;
   }

   *pofstream << "  </constraints>" << endl;
   *pofstream << endl;
   *pofstream << "  <enum_vals>" << endl;

   pEnum = DeviceDither::getEnumeration ();
   while (pEnum->hasMoreElements ())
   {
      char *pszID = (char *)pEnum->nextElement ();

      //raharper@us.ibm.com took out the ev/omni-
      *pofstream << "    <enum_val id='" << pszID << "'>" << endl;
      *pofstream << "      <ev_longname><en>" << DeviceDither::getName (pszID) << "</en></ev_longname>" << endl;
      *pofstream << "      <ev_shortname><en>" << pszID << "</en></ev_shortname>" << endl;
      *pofstream << "      <ev_driverval>" << pszID << "</ev_driverval>" << endl;
      *pofstream << "      <constraints>" << endl;
      *pofstream << "        <!-- Assume the option doesn't apply... -->" << endl;
      *pofstream << "        <constraint sense='false'>" << endl;
      *pofstream << "          <driver>omni</driver>" << endl;
      *pofstream << "        </constraint>" << endl;

      for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
      {
         string *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

         *pofstream << "        <constraint sense='true'>" << endl;
         *pofstream << "          <driver>omni</driver>" << endl;
         *pofstream << "          <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
         *pofstream << "        </constraint>" << endl;
      }

      *pofstream << "      </constraints>" << endl;
      *pofstream << "    </enum_val>" << endl;
   }

   *pofstream << "  </enum_vals>" << endl;
   *pofstream << "</option>" << endl;

   /**
    * Make Orientation section
    *
    * List orientations
    */
   ofstream ofstreamOrientation ("foomatic-db/db/source/opt/omni-orientation.xml");
   pofstream = &ofstreamOrientation;

   cout << "Generating omni-orientation.xml" << endl;

   *pofstream << "<option type=\"enum\" id=\"opt/omni-orientation\">" << endl;
   *pofstream << "  <!-- A multilingual <comments> block can appear here, too;" << endl;
   *pofstream << "       it should be treated as documentation for the user. -->" << endl;
   *pofstream << "  <arg_longname>" << endl;
   // Edited by raharper@us.ibm.com, changing the output to match omni driver options
   *pofstream << "   <en>Orientation</en>" << endl;
   *pofstream << "  </arg_longname>" << endl;
   *pofstream << "  <arg_shortname>" << endl;
   // Edited by raharper@us.ibm.com, changing the output to match omni driver options
   *pofstream << "   <en>Orientation</en><!-- backends only know <en> shortnames! -->" << endl;
   *pofstream << "  </arg_shortname>" << endl;
   *pofstream << "  <arg_execution>" << endl;
   *pofstream << "   <arg_order>100</arg_order>" << endl;
   *pofstream << "   <arg_spot>A</arg_spot>" << endl;
   *pofstream << "   <arg_substitution />" << endl;
   // Edited by raharper@us.ibm.com, changing the output to match omni driver options
   *pofstream << "   <arg_proto> orientation=%s</arg_proto>" << endl;
   *pofstream << "  </arg_execution>" << endl;
   *pofstream << endl;

   *pofstream << "  <constraints>" << endl;

   for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
   {
      Device            *pDevice           = pohDevices[iCurrentDevice].pDevice;
      DeviceOrientation *pDT               = pDevice->getCurrentOrientation ();
      string            *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

      *pofstream << "    <constraint sense='true'>" << endl;
      *pofstream << "      <driver>omni</driver>" << endl;
      *pofstream << "      <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
      // raharper@us.ibm.com, changing what the defval output looks like
      *pofstream << "      <arg_defval>" << DeviceOrientation::IDToName (pDT->getID ()) << "</arg_defval>" << endl;
      *pofstream << "    </constraint>" << endl;
   }

   *pofstream << "  </constraints>" << endl;
   *pofstream << endl;
   *pofstream << "  <enum_vals>" << endl;

   for (int iOrient = DeviceOrientation::ORIENTATION_NONE; iOrient < DeviceOrientation::COUNT; iOrient++)
   {
      bool fFirstTime = true;

      for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
      {
         Device            *pDevice           = pohDevices[iCurrentDevice].pDevice;
         DeviceOrientation *pDT               = pDevice->getCurrentOrientation ();
         string            *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

         if (pDT->isOrientationSupported (iOrient))
         {
            if (fFirstTime)
            {
               fFirstTime = false;
               //raharper@us.ibm.com took out the ev/omni-
               *pofstream << "    <enum_val id='" << DeviceOrientation::IDToName (iOrient) << "'>" << endl;
               *pofstream << "      <ev_longname><en>" << DeviceOrientation::getName (iOrient) << "</en></ev_longname>" << endl;
               *pofstream << "      <ev_shortname><en>" << DeviceOrientation::IDToName (iOrient) << "</en></ev_shortname>" << endl;
               *pofstream << "      <ev_driverval>" << DeviceOrientation::IDToName (iOrient) << "</ev_driverval>" << endl;
               *pofstream << "      <constraints>" << endl;
               *pofstream << "        <!-- Assume the option doesn't apply... -->" << endl;
               *pofstream << "        <constraint sense='false'>" << endl;
               *pofstream << "          <driver>omni</driver>" << endl;
               *pofstream << "        </constraint>" << endl;
            }

            *pofstream << "        <constraint sense='true'>" << endl;
            *pofstream << "          <driver>omni</driver>" << endl;
            *pofstream << "          <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
            *pofstream << "        </constraint>" << endl;
         }
      }

      if (!fFirstTime)
      {
         *pofstream << "      </constraints>" << endl;
         *pofstream << "    </enum_val>" << endl;
      }
   }

   *pofstream << "  </enum_vals>" << endl;
   *pofstream << "</option>" << endl;

    /*******
    * Make Option section
    *
    * List Omni driver options
    ******
   
   // Here is what needs to be done.  There are additional options, that are in the Omni driver
   // that each printer supports, or rather, Omni does for each printer, regardless.  For each
   // of these options (to be defined in a file), this method must generate an omni-<option>.xml
   // describing the properites of the option in question.  The way this will work is the Foomatic.cpp
   // code will look for a specific file ( there will be a command line override added for the
   // user to specify by hand the list of options they want generated) and Foomatic will read
   // the options from the file and generate an xml file for each of the options in the file
   //
   // The reading of the file will be one method, and then there will be a skeleton function
   // that will take the name of the option and generate the omni-<option>.xml file with
   // the specifics.  Each option will have a boolean flag, either on or off.  The only thing left
   // is to determine whether the default it on or off, that can be specified in the file.
   //
   // 
   // Variables so far:
   // String OptionLongname
   // String OptionShortname
   // String prototype

   ofstream ofstreamOption ("foomatic-db/db/source/opt/omni-myoption.xml");
   pofstream = &ofstreamOption;
   string OptionLongname("OptionLongname");
   string OptionShortname("OptionShortname");
   string prototype("myoption");

   cout << "Generating omni-myoption.xml" << endl;

   *pofstream << "<option type=\"enum\" id=\"opt/omni-myoption\">" << endl;
   *pofstream << "  <!-- A multilingual <comments> block can appear here, too;" << endl;
   *pofstream << "       it should be treated as documentation for the user. -->" << endl;
   *pofstream << "  <arg_longname>" << endl;
   *pofstream << "   <en>" << OptionLongname << "</en><!-- backends only know <en> shortnames! -->" << endl;
   *pofstream << "  <arg_shortname>" << endl;
   *pofstream << "   <en>" << OptionShortname << "</en>" << endl;
   *pofstream << "  </arg_shortname>" << endl;
   *pofstream << "  <arg_execution>" << endl;
   *pofstream << "   <arg_order>100</arg_order>" << endl;
   *pofstream << "   <arg_spot>A</arg_spot>" << endl;
   *pofstream << "   <arg_substitution />" << endl;
   *pofstream << "   <arg_proto> " << prototype << "=%s</arg_proto>" << endl;
   *pofstream << "  </arg_execution>" << endl;
   *pofstream << endl;

   // Put in default value for each printer
   // Note to self:  find out whether this option defaults to 1 or 0
   *pofstream << "  <constraints>" << endl;

   for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
   {
      //Device            *pDevice           = pohDevices[iCurrentDevice].pDevice;
      //DeviceOrientation *pDT               = pDevice->getCurrentOrientation ();
      string            *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

      *pofstream << "    <constraint sense='true'>" << endl;
      *pofstream << "      <driver>omni</driver>" << endl;
      *pofstream << "      <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
      *pofstream << "      <arg_defval>" << 1 << "</arg_defval>" << endl;
      *pofstream << "    </constraint>" << endl;
   }

   *pofstream << "  </constraints>" << endl;
   *pofstream << endl;
   *pofstream << "  <enum_vals>" << endl;


   // Here is where we print out what printers support what options.  This can be stripped down to just
   // producing what the option is, and that all devices support the current option.

   // Iterate over all the possible option values (0 or 1)
   for (int iCurrent = 0; iCurrent < 2; iCurrent++)
   {
      bool fFirstTime = true;
      for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
      {
       //  Device            *pDevice           = pohDevices[iCurrentDevice].pDevice;
       //  DeviceOrientation *pDT               = pDevice->getCurrentOrientation ();
         string            *pstringDeviceName = pohDevices[iCurrentDevice].pstringDeviceName;

         if(iCurrent) {
            if (fFirstTime)
            {
               fFirstTime = false;
               *pofstream << "    <enum_val id='" << OptionShortname << "'>" << endl;
               *pofstream << "      <ev_longname><en>" << OptionLongname << "</en></ev_longname>" << endl;
               *pofstream << "      <ev_shortname><en>" << OptionShortname << "</en></ev_shortname>" << endl;
               *pofstream << "      <ev_driverval>" << iCurrent << "</ev_driverval>" << endl;
               *pofstream << "      <constraints>" << endl;
               *pofstream << "        <!-- Assume the option doesn't apply... -->" << endl;
               *pofstream << "        <constraint sense='false'>" << endl;
               *pofstream << "          <driver>omni</driver>" << endl;
               *pofstream << "        </constraint>" << endl;
            }

            *pofstream << "        <constraint sense='true'>" << endl;
            *pofstream << "          <driver>omni</driver>" << endl;
            *pofstream << "          <printer>printer/" << *pstringDeviceName << "</printer>" << endl;
            *pofstream << "        </constraint>" << endl;

         }
      }
      if (!fFirstTime)
      {
         *pofstream << "      </constraints>" << endl;
         *pofstream << "    </enum_val>" << endl;
      }
   }

   *pofstream << "  </enum_vals>" << endl;
   *pofstream << "</option>" << endl;

ENDOFOPTIONS   
   */

   // Clean up
   for (iCurrentDevice = 0; iCurrentDevice < iNumDevices; iCurrentDevice++)
   {
      delete pohDevices[iCurrentDevice].pstringDeviceName;
      delete pohDevices[iCurrentDevice].pDevice;
      g_module_close (pohDevices[iCurrentDevice].hmodDevice);
   }

   delete[] pohDevices;

   return 0;
}
